/**
 * Eine einfache Tcl-Erweiterung um auf die parralele Schnittstelle
 * unter Unix zu Schreiben. Die Erweiterung bentigt Root-Rechte,
 * sie benutzt inb and outb auf Port 888.
 * 
 */

#include <asm/io.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <tcl.h>

/* Vorwrtsreferenz */
int parcon_Cmd(ClientData cdata, Tcl_Interp *interp, int objc,
      Tcl_Obj * CONST objv[]);

/*
 * Initialisiert die Erweiterung und berprft die Rechte
 */
int Parcon_Init(Tcl_Interp *interp) 
{

#ifdef USE_TCL_STUBS
  if (Tcl_InitStubs(interp, "8.1", 0) == 0L) {
    return TCL_ERROR;
  }
#endif

  /* Kann die parallel Schnittstelle geffnet werden  ? */
  if( ioperm(888,1,1) ) {
    /* Fehlermeldung erzeugen */
    Tcl_Obj *retval =  Tcl_NewStringObj( "Parallele Schnittstelle kann nicht geffnet werden.", -1);
    Tcl_AppendStringsToObj( retval, "\nBitte als Root benutzen", (char*) NULL );

    Tcl_SetObjResult( interp, retval);

    return TCL_ERROR;
  }

  /* Kommando erzeugen */
  Tcl_CreateObjCommand(interp, "parcon", parcon_Cmd, NULL, NULL);

  return TCL_OK;
}

/**
 * Das Kommando selber
 */
int parcon_Cmd(ClientData cdata, Tcl_Interp *interp, int objc,
	       Tcl_Obj * CONST objv[]) 
{

  /** entweder kein Argument oder eines */
  if ( objc > 2 ) {
    Tcl_WrongNumArgs( interp, 1, objv, "?bitmap?" );
    return TCL_ERROR;
  }

  /* Eingabe berprfen und Status setzen*/
  if ( objc == 2 ) {
    
    // Lnge prfen
    if ( Tcl_GetCharLength( objv[1]) != 8 ) {
      Tcl_Obj *retval = 
	Tcl_NewStringObj( "Falsche Eingabe, bentige 8bit ( \"00101010\" )", -1);
      Tcl_SetObjResult( interp, retval);
      
      return TCL_ERROR;
    }

    char *Bitmap = Tcl_GetString (objv[1]);
    int wert = 0, i;

    //
    // Umwandlung bitmap in eine Integerwert,
    // entsprechend dem Vorschlag von Stefan Schober
    //
    for (i = 0; i < 8; i++) {
      wert <<= 1;
      if (Bitmap[i] == '1') {
	wert |= 1;
      }
    }
    //printf("werte %i\n", wert);
    /* Parallelport setzen*/
    outb(wert,888);  
  }

  /* Parallelport auslesen */
  unsigned char bitmap = inb(888);

  /* .. und Ergebniss formatieren */
  Tcl_Obj *result = Tcl_NewStringObj("", -1);
  int i;
  for ( i=7; i>=0; i-- ) { 
    Tcl_AppendStringsToObj( result, ( bitmap & (1<<i) ) ? "1" : "0" , (char*) NULL ); 
  } 
  /* Ergebniss setzen */
  Tcl_SetObjResult(interp, result);

  return TCL_OK;
}
